@@ -63,7 +63,7 @@ extern int is_read_only();
 extern void set_device_ro();
 
 extern void rd_load();
-extern long rd_init();
+/* extern long rd_init(); */
 
 extern unsigned long xd_init();
 
