@@ -36,10 +36,20 @@
 #include <linuxmt/fs.h>
 /*#include <linux/signal.h> */
 #include <linuxmt/string.h>
-#include <arch/segment.h>
+#include <linuxmt/mm.h>
 
+#include <arch/segment.h>
+/* Uncomment this if your floppy drive(s) can't be properly recognized */
+/* Search for HARDCODED and adjust ndrives and drive_info[] to match your */
+/* system */
+
+/* #define HARDCODED /**/
+
+#include <arch/system.h>
 #define MAJOR_NR BIOSHD_MAJOR
 #define BIOSDISK
+#define NR_HD ((drive_info[1].heads)?2:((drive_info[0].heads)?1:0))
+#define put_user(val,ptr) pokew(current->t_regs.ds,ptr,val)
 #include "blk.h"
 
 static int bioshd_ioctl();
@@ -61,12 +71,12 @@ static struct file_operations bioshd_fops =
 	NULL,			/* revalidate */
 };
 
-static struct wait_queue *busy_wait = NULL;
+/* static struct wait_queue *busy_wait = NULL; */
 static struct wait_queue dma_wait;
 static int dma_avail = 1;
 
 static int bioshd_initialized = 0;
-static int force_bioshd;
+/* static int force_bioshd; */
 static int revalidate_hddisk();
 static struct drive_infot
 {
@@ -85,11 +95,13 @@ struct drive_infot fd_types[] = {
 	{80, 36, 2, 4},
 };
 
+/* This makes probing order more logical */
+/* and avoids few senseless seeks in some cases */
+static int probe_order[5] = {0, 2, 1, 3, 4};
+
 static struct hd_struct hd[4 << 6];
-static char busy[4] =
-{0,};
-static int access_count[4] =
-{0,};
+/* static char busy[4] = {0,}; */
+static int access_count[4] = {0,};
 static unsigned char hd_drive_map[4] =
 {
 	0x80, 0x81,		/* hda, hdb */
@@ -98,8 +110,7 @@ static unsigned char hd_drive_map[4] =
 
 static void bioshd_geninit();
 
-static int bioshd_sizes[4 << 6] =
-{0,};
+static int bioshd_sizes[4 << 6] = {0,};
 
 static struct gendisk bioshd_gendisk =
 {
@@ -119,15 +130,16 @@ static struct gendisk bioshd_gendisk =
 /* This function checks to see which hard drives are active and sets up the
  * drive_info[] table for them.  Ack this is darned confusing... */
 
+#ifdef CONFIG_BLK_DEV_BHD
 int bioshd_gethdinfo()
 {
 	int drive, ndrives = 0;
 
 	for (drive = 0; drive <= 1; drive++) {
 		ndrives++;
-		BD_AX = 0x0800;
+		BD_AX = BIOSHD_DRIVE_PARMS;
 		BD_DX = drive + 0x80; 
-		BD_IRQ = 0x13;
+		BD_IRQ = BIOSHD_INT;
 		call_bios();
 		if ((BD_AX != 0x100) && (!CARRY_SET)) {
 			drive_info[drive].cylinders = ((BD_CX >> 8) & 255);
@@ -141,45 +153,96 @@ int bioshd_gethdinfo()
 	}
 	return ndrives;
 }
-
+#endif
+#ifdef CONFIG_BLK_DEV_BFD
 int bioshd_getfdinfo()
 {
-	int i, ndrives;
+#ifdef HARDCODED
+	/* Set this to match your system */
+	/* Currently it's set to two drive system, 1.44MB as /dev/fd0 */
+	/* and 1.2 MB as /dev/fd1 */
+
+	/* ndrives is number of drives in your system (either 0, 1 or 2) */
+
+	int ndrives = 2;
+
+	/* drive_info[] should be set *only* for existing drives; */
+	/* comment out drive_info lines if you don't need them (e.g.*/
+	/* you have less than 2 drives) */
+
+	/* Enter type 4 in fd_types' brackets for unknown drive type */
+	/* Otherwise use table below */
+
+	/* Floppy drive type table:
+		Type:	Format:
+		0	360 KB
+		1	1.2 MB
+		2	720 KB
+		3	1.44 MB
+		4	2.88 MB/Unknown
+	*/
+
+	/* /dev/fd0 */
+	drive_info[2] = fd_types[3];
+
+	/* /dev/fd1 */
+	drive_info[3] = fd_types[1];
+
+	/* That's it .. you're done :-) */
+
+	/* Warning: drive will be reported as 2880 KB at bootup if you've */
+	/* set it as unknown (4). Floppy probe will detect correct floppy */
+	/* format at each change so don't bother with that */
+
+#else
+
+	int drive, ndrives;
 
 	/* We get the # of drives from the BPB, which is PC-friendly */
 	ndrives = (peekb(0x40, 0x10) >> 6) + 1;
 	printk("doshd: found %d floppy drives\n", ndrives);
-	for (i = 0; i < ndrives; i++) {
+
+	for (drive = 0; drive < ndrives; drive++) {
 		BD_AX = 0x800;
-		BD_DX = i;
+		BD_DX = drive;
 		BD_BX = 0;
-		BD_IRQ = 0x13;
-		printk("%x\n", BD_AX);
+		BD_IRQ = BIOSHD_INT;
+/*		printk("%x\n", BD_AX); */
 		call_bios();
-		printk("%x\n", BD_AX);
-		if ((!CARRY_SET) && (BD_AX != 0x100)) { 
+/*		printk("%x\n", BD_AX); */
+		if ((!CARRY_SET) && (BD_AX != 0x100) && (arch_cpu > 5)) { 
+			/* Some XT's return strange results - Al */
+			/* The arch_cpu is a safety check */
 			/* AT archecture, drive type in DX */
-			printk("drive %d is %d\n", i, BD_BX);
-			drive_info[i+2] = fd_types[BD_BX - 1];
+			printk("drive %d is %d\n", drive, BD_BX);
+			drive_info[drive+2] = fd_types[BD_BX - 1];
 		} else {	
-			/* PC arch, assume 360K */
-			printk("drive %d is 360k\n", i);
-			drive_info[i+2] = fd_types[0];
+			/* Cannot be determined correctly */
+			/* Type 4 should work on all systems */
+			printk("drive %d type unknown\n", drive);
+			drive_info[drive+2] = fd_types[4];
 		}
 	}
+#endif /* HARDCODED */
+
 	return ndrives;
 }
+#endif /* CONFIG_BLK_DEV_BFD */
 
-static void
- bioshd_release(inode, filp)
+static void bioshd_release(inode, filp)
     struct inode *inode;
     struct file *filp;
 {
 	int target;
-	sync_dev(inode->i_rdev);
+	kdev_t	dev = inode->i_rdev;
+	sync_dev(dev);
 
-	target = DEVICE_NR(inode->i_rdev);
+	target = DEVICE_NR(dev);
 	access_count[target]--;
+	if ((target >= 2) && (access_count[target] == 0)) {
+		invalidate_inodes(dev);
+		invalidate_buffers(dev);
+	}
 	return;
 }
 
@@ -201,8 +264,7 @@ static void reset_bioshd(minor)
 	return;
 }
 
-static int
- bioshd_open(inode, filp)
+static int bioshd_open(inode, filp)
     struct inode *inode;
     struct file *filp;
 {
@@ -223,21 +285,20 @@ static int
 
 	/* Wait until it's free */
 
-	while (busy[target])
-		sleep_on(&busy_wait);
+/*	while (busy[target])
+		sleep_on(&busy_wait); */
 
 	/* Register that we're using the device */
 
 	access_count[target]++;
 
-	/* Check the disk type */
+	/* Check for disk type */
 
-	/* Currently, this can only differentiate between 1.4MB and 720kB
-	   discs. It does this by trying to read sector 10 of track 0. If
-	   it succeeds, it assumes that there are 18 sectors, if not 9. */
+	/* I guess it works now as it should. Tested under dosemu with 729Kb,
+	1.2 MB and 1.44 MB floppy image and works fine - Blaz Antonic */
 
-	if (target >= 2)
-	{
+#ifdef CONFIG_BLK_DEV_BFD
+	if (target >= 2) {
 		int bufseg;
 		int count;
 		int i;
@@ -252,49 +313,52 @@ static int
 		bufseg = 0x800;
 
 		count = 0;
-		for (i = 0; i <= fdtype; i++)
-                {
-		   count = 0;
+		for (i = 0; i <= probe_order[fdtype]; i++)
+		{
+			count = 0;
  
-		   while (count < 3)
-		   {
-/*			printk("type %d count %d\n", i, count);
-*/			/* BIOS read sector */
-
-			BD_IRQ = BIOSHD_INT;
-			BD_AX = BIOSHD_READ | 1;	/* Read 1 sector */
-			BD_BX = 0;	/* Seg offset = 0 */
-			BD_ES = bufseg;		/* Segment to read to */
-			BD_CX = (fd_types[i].cylinders<<8)|1;
-			BD_CX += (0<<8)|fd_types[i].sectors;
-			BD_DX = (0 << 8) | hd_drive_map[target];	/* Head 0, drive number */
-			BD_FL = 0;
-
-			sti();
-			call_bios();
-
-			if (CARRY_SET)
+			while (count < MAX_ERRS)
 			{
-				if (((BD_AX >> 8) == 0x04) && (count == 2)) 
-				                         /* Sector not found */
+/*				printk("type %d count %d\n", probe_order[i], count); */
+				/* BIOS read sector */
+
+				BD_IRQ = BIOSHD_INT;
+				BD_AX = BIOSHD_READ | 1;	/* Read 1 sector */
+				BD_BX = 0;	/* Seg offset = 0 */
+				BD_ES = bufseg;		/* Segment to read to */
+				BD_CX = ((fd_types[probe_order[i]].cylinders-40)<<8)|fd_types[probe_order[i]].sectors;
+				BD_DX = (0 << 8) | hd_drive_map[target];	/* Head 0, drive number */
+				BD_FL = 0;
+
+				sti();
+				call_bios();
+
+				if (CARRY_SET)
+				{
+					if (((BD_AX >> 8) == 0x04) && (count == MAX_ERRS - 1)) 
+						break; /* Sector not found */
+
+					reset_bioshd(hd_drive_map[target]);
+					count++;
+				} else {
+					drive_info[target].cylinders = fd_types[probe_order[i]].cylinders;
+					drive_info[target].sectors = fd_types[probe_order[i]].sectors;
 					break;
-
-				reset_bioshd(hd_drive_map[target]);
-				count++;
-			} else
-			{
-				drive_info[target].cylinders = fd_types[i].cylinders;
-				drive_info[target].sectors = fd_types[i].sectors;
-				break;
-			}   
-                }
+				}   
+			}
 		
+		}
+		/* DMA code belongs out of the loop. */
+		dma_avail = 1;
+		wake_up(&dma_wait);
+
+		/* I moved this out of for loop to prevent trashing the screen
+		with seducing (repeating) probe messages about disk types */
+
 		if (drive_info[target].cylinders == 0 || drive_info[target].sectors == 0)
 			printk("hd: Autoprobe failed!\n");
-
-		printk("hd: disc in /dev/fd%d probably has %d sectors and %d cylinders\n", target % 2,
-		       drive_info[target].sectors, drive_info[target].cylinders);
-	}
+		else
+			printk("hd: disc in /dev/fd%d probably has %d sectors and %d cylinders\n", target % 2, drive_info[target].sectors, drive_info[target].cylinders);
 
 /*
  *	This is not a bugfix, hence no code, but coders should be aware
@@ -310,8 +374,7 @@ static int
  *	You may have to copy dpb to RAM as the original is in ROM.
  */
 	}
-	dma_avail = 1;
-	wake_up(&dma_wait);
+#endif
 	return 0;
 }
 
@@ -325,8 +388,13 @@ void init_bioshd()
 
 	printk("hd Driver Copyright (C) 1994 Yggdrasil Computing, Inc.\nExtended and modified for Linux8086 by Alan Cox.\n");
 
+#ifdef CONFIG_BLK_DEV_BFD
 	bioshd_getfdinfo();
+#endif
+#ifdef CONFIG_BLK_DEV_BHD
 	bioshd_gethdinfo();
+#endif
+
 	for (i = 0; i <= 3; i++) {
 		if (drive_info[i].heads) {count++; if (i <= 1) hdcount++;} 
 	}
@@ -356,8 +424,8 @@ void init_bioshd()
 	if (i == 0)
 	{
 		blk_dev[MAJOR_NR].request_fn = DEVICE_REQUEST;
-		blksize_size[MAJOR_NR] = 1024;
-		read_ahead[MAJOR_NR] = 2;
+		/* blksize_size[MAJOR_NR] = 1024; */ /* Currently unused */
+		/* read_ahead[MAJOR_NR] = 2; */ /* Currently unused */
 		if (gendisk_head == NULL)
 		{
 			bioshd_gendisk.next = gendisk_head;
@@ -378,8 +446,32 @@ static int bioshd_ioctl(inode, file, cmd, arg)
     struct inode *inode;
     struct file *file;
     unsigned int cmd;
-    unsigned long arg;
+    unsigned int arg;
 {
+	struct hd_geometry *loc = (struct hd_geometry *) arg; /**/
+	int dev, err;
+	unsigned long flags;
+
+	if ((!inode) || !(inode->i_rdev))
+		return -EINVAL;
+	dev = DEVICE_NR(inode->i_rdev);
+	if (dev >= NR_HD)
+		return -ENODEV;
+
+	switch(cmd) {
+		case HDIO_GETGEO:
+			err = verify_area(VERIFY_WRITE,arg,sizeof(*loc));
+			if (err)
+				return err;
+			put_user(drive_info[dev].heads, (char *) &loc->heads);
+			put_user(drive_info[dev].sectors, (char *) &loc->sectors);
+			put_user(drive_info[dev].cylinders, (short *) &loc->cylinders);
+			put_user(hd[MINOR(inode->i_rdev)].start_sect, (long *) &loc->start);
+
+			return 0;
+			break;
+	}
+		
 	return -EINVAL;
 }
 
@@ -396,14 +488,14 @@ static void do_bioshd_request()
 	char *buff;
 	int minor;
 	int errs;
-	int part;
+/*	int part; */
 	int drive;
 
 	while (1)
 	{
 	      next_block:
 
-		/* make sure we have a valid request */
+		/* make sure we have a valid request *//*Done by INIT_REQUEST*/
 		if (!CURRENT || CURRENT->dev < 0)
 			return;
 
@@ -420,7 +512,7 @@ static void do_bioshd_request()
 			continue;
 		}
 		minor = MINOR(CURRENT->dev);
-		part = minor & ((1 << 6) - 1);
+/*		part = minor & ((1 << 6) - 1); */
 		drive = minor >> 6;
 
 		/* make sure it's a disk we are dealing with. */
@@ -514,7 +606,7 @@ static void do_bioshd_request()
 	}
 }
 
-#define DEVICE_BUSY busy[target]
+/* #define DEVICE_BUSY busy[target] */
 #define USAGE access_count[target]
 #define CAPACITY ((long)drive_info[target].heads*drive_info[target].sectors*drive_info[target].cylinders)
 /* We assume that the the bios parameters do not change, so the disk capacity
@@ -530,6 +622,7 @@ static void do_bioshd_request()
  * usage == 1 (we need an open channel to use an ioctl :-), so this
  * is our limit.
  */
+#if 0 /* Currently not used, removing for size. */
 static int revalidate_hddisk(dev, maxusage)
     int dev;
     int maxusage;
@@ -576,6 +669,7 @@ static int revalidate_hddisk(dev, maxusage)
 	wake_up(&busy_wait);
 	return 0;
 }
+#endif
 
 static void bioshd_geninit()
 {
@@ -595,5 +689,5 @@ static void bioshd_geninit()
 			hd[i].nr_sects = 0;
 		}
 	}
-	blksize_size[MAJOR_NR] = 1024;
+	/* blksize_size[MAJOR_NR] = 1024; */ /* Currently unused */
 }
