@@ -267,7 +267,7 @@ unsigned char Key;
    }
    return 0;
 }
-
+#if 0
 int GetQueue()
 {
    return chq_getch(&ttys[0].inq, 0, 0);
@@ -278,12 +278,12 @@ unsigned State;
 {
    /* Set the keyboard leds according to the locks in 'State' */
 }
-
+#endif
 /*
  *      Busy wait for a keypress in kernel state for bootup/debug.
  */
 
 int wait_for_keypress()
 {
-	return GetQueue();
+	return chq_getch(&ttys[0].inq, 0, 1);
 }
