@@ -93,6 +93,8 @@ void *regs;
 		irq->handler(i,regs);
 		lastirq = -1;
 	}
+	if (i > 1)
+		printk("IRQ no.%d received.\n",i);
 	else if(irq->flags&IRQF_BAD)
 	{
 		if(irq>16)
@@ -106,13 +108,17 @@ void *regs;
  *	Low level IRQ control.
  */
  
-unsigned long __save_flags()
-{
+/*unsigned long __save_flags()
+{*/
 #asm
+	.globl ___save_flags
+	.text
+___save_flags:
 	pushf
 	pop ax
+	ret
 #endasm
-}
+/*}*/
 
 void restore_flags(flags)
 unsigned long flags;
@@ -248,14 +254,14 @@ void init_IRQ()
 	 *	Set off the initial timer interrupt handler
 	 */
 	 
-	if(request_irq(0, timer_tick, 0, "timer"))
+	if(request_irq(0, timer_tick, 0L, "timer"))
 		panic("Unable to get timer");
 	 	
 	/*
 	 *	Set off the initial keyboard interrupt handler
 	 */
 
-	if(request_irq(1, keyboard_irq, 0, "keyboard"))
+	if(request_irq(1, keyboard_irq, 0L, "keyboard"))
 		panic("Unable to get keyboard");
 	/*
 	 *	Enable the drop through interrupts.
@@ -266,7 +272,7 @@ void init_IRQ()
 	enable_irq(5);		/* XT ST506 */
 	enable_irq(2);		/* Cascade */
 	enable_irq(6);		/* Floppy */
-	
+
 	/*
 	 *	All traps are bad initially. One day IRQ's will be.
 	 */
