@@ -28,7 +28,7 @@ int *end;
 	mov ax, ds
 	mov [bx+4], ax
 	mov [bx+10], dx
-	mov ds, ax
+!	mov ds, ax
 #endasm
 	arch_segs.lowss = arch_segs.endss;
 	/*
