@@ -51,42 +51,41 @@ int wr;
 		 *	of a whole block to avoid a read of the data.
 		 */
 		if(wr&&chars==BLOCK_SIZE)
-			bh = getblk(dev,block,BLOCK_SIZE);
-		else
-			bh = bread(dev,block,BLOCK_SIZE);
+			bh = getblk(dev,block);
+		else 
+			bh = bread(dev,block);
 		block++;
 		if (!bh)
 			return written?written:-EIO;
-		p = offset + bh->b_data;
-		offset = 0;
+/*		p = offset + bh->b_data; */
+/*		offset = 0; */
 		filp->f_pos += chars;
 		written += chars;
 		count -= chars;
 		/*
 		 *	What are we doing ?
 		 */
+		map_buffer(bh);
+		p = offset + bh->b_data;
 		if(wr)
 		{
 			/*
 			 *	Alter buffer, mark dirty
 			 */
-			map_buffer(bh);
 			memcpy_fromfs(p,buf,chars);
 			bh->b_uptodate = 1;
-			bh->b_dirty = 0;
-		}
-		else
-		{
+			bh->b_dirty = 1;
+		} else {
 			/*
 			 *	Empty buffer data. Buffer unchanged
 			 */
-			map_buffer(bh);
 			memcpy_tofs(buf,p,chars);
 		}
 		/*
 		 *	Move on and release buffer
 		 */
-		p += chars;
+/*		p += chars; */
+		offset = 0;
 		buf += chars;
 		if(wr)
 		{
@@ -97,10 +96,10 @@ int wr;
 			wait_on_buffer(bh);
 			if (!bh->b_uptodate)
 				write_error=1;
-			if(write_error)
-				break;
 		}
 		unmap_brelse(bh);
+		if(write_error)
+			break;
 	}
 	if(write_error)
 		return -EIO;
