@@ -12,17 +12,17 @@ struct buffer_head *bh;
 	if(bh)
 		__brelse(bh);
 }
-
+/*
 extern void bforget(bh)
 struct buffer_head *bh;
 {
 	if(bh)
 		__bforget(bh);
 }
-
+ */
 extern struct inode *iget(sb,nr)
 struct super_block *sb;
-int nr;
+long nr;
 {
 	return __iget(sb,nr,1);
 }
