@@ -11,6 +11,7 @@
 #include <linuxmt/fs.h>
 
 #include <linuxmt/minix_fs.h>
+#include <linuxmt/romfs_fs.h>
 #include <linuxmt/major.h>
 
 void fs_init()
@@ -18,4 +19,7 @@ void fs_init()
 #ifdef CONFIG_MINIX_FS
 	init_minix_fs();
 #endif
+#ifdef CONFIG_ROMFS_FS
+	init_romfs_fs();
+#endif
 }
