@@ -539,7 +539,7 @@ struct inode * get_pipe_inode()
 
 struct inode *__iget(sb,nr,crossmntp)
 struct super_block * sb;
-int nr;
+long nr;
 int crossmntp;
 {
 	static struct wait_queue * update_wait = NULL;
@@ -547,7 +547,7 @@ int crossmntp;
 	struct inode * inode;
 	struct inode * empty = NULL;
 
-	printd_iget("iget called(%x, %d, %d)\n", sb, nr, crossmntp);
+	printd_iget3("iget called(%x, %d, %d)\n", sb, nr, crossmntp);
 	if (!sb)
 		panic("VFS: iget with sb==NULL");
 	h = hash(sb->s_dev, nr);
@@ -559,7 +559,7 @@ repeat:
 		h->updating++;
 		printd_iget("iget: getting an empty inode...\n");
 		empty = get_empty_inode();
-		printd_iget("iget: got one... (%x)!\n", empty);
+		printd_iget1("iget: got one... (%x)!\n", empty);
 		if (!--h->updating)
 			wake_up(&update_wait);
 		if (empty)
