@@ -37,8 +37,7 @@ unsigned int numbits;
 		} else {
 			int tmp;
 			end = numbits >> 3;
-			numbits &= 0x7;
-			tmp = bh->b_data[end] & ((1<<numbits)-1);
+			tmp = bh->b_data[end] & ((1<<(numbits&7))-1);
 			sum += nibblemap[tmp&0xf] + nibblemap[(tmp>>4)&0xf];
 			numbits = 0;
 		}  
@@ -66,7 +65,7 @@ unsigned long block;
 		printk("trying to free block not in datazone\n");
 		return;
 	}
-	bh = get_hash_table(sb->s_dev,block,BLOCK_SIZE);
+	bh = get_hash_table(sb->s_dev,block);
 	if (bh)
 		bh->b_dirty = 0;
 	brelse(bh);
@@ -124,7 +123,7 @@ repeat:
 	    j >= sb->u.minix_sb.s_nzones)
 		return 0;
 	/* WARNING: j is not converted, so we have to do it */
-	if (!(bh = getblk(sb->s_dev, (long)j, 1024))) {
+	if (!(bh = getblk(sb->s_dev, (long)j))) {
 		printk("new_block: cannot get block");
 		return 0;
 	}
@@ -143,6 +142,7 @@ struct super_block *sb;
 		 << sb->u.minix_sb.s_log_zone_size;
 }
 
+static char *fi_name = "free_inode";
 void minix_free_inode(inode)
 struct inode * inode;
 {
@@ -152,38 +152,40 @@ struct inode * inode;
 	if (!inode)
 		return;
 	if (!inode->i_dev) {
-		printk("free_inode: inode has no device\n");
+		printk("%s: inode has no device\n",fi_name);
 		return;
 	}
 	if (inode->i_count != 1) {
-		printk("free_inode: inode has count=%d\n",inode->i_count);
+		printk("%s: inode has count=%d\n",fi_name,inode->i_count);
 		return;
 	}
 	if (inode->i_nlink) {
-		printk("free_inode: inode has nlink=%d\n",inode->i_nlink);
+		printk("%s: inode has nlink=%d\n",fi_name,inode->i_nlink);
 		return;
 	}
 	if (!inode->i_sb) {
-		printk("free_inode: inode on nonexistent device\n");
+		printk("%s: inode on nonexistent device\n",fi_name);
 		return;
 	}
 	if (inode->i_ino < 1 || inode->i_ino >= inode->i_sb->u.minix_sb.s_ninodes) {
-		printk("free_inode: inode 0 or nonexistent inode\n");
+		printk("%s: inode 0 or nonexistent inode\n",fi_name);
 		return;
 	}
 	ino = inode->i_ino;
 	if (!(bh=inode->i_sb->u.minix_sb.s_imap[ino >> 13])) {
-		printk("free_inode: nonexistent imap in superblock\n");
+		printk("%s: nonexistent imap in superblock\n",fi_name);
 		return;
 	}
 	map_buffer(bh);
 	clear_inode(inode);
-	if (!clear_bit(ino & 8191, bh->b_data))
-/*		printk("free_inode: bit %ld already cleared.\n",ino); */
+	if (!clear_bit(ino & 8191, bh->b_data)) {
+/*		printk("%s: bit %ld already cleared.\n",ino); */
+	}
 	mark_buffer_dirty(bh, 1);
 	unmap_buffer(bh);
 }
 
+#ifndef CONFIG_FS_RO
 struct inode * minix_new_inode(dir)
 struct inode * dir;
 {
@@ -237,6 +239,7 @@ struct inode * dir;
 	printk("new_inode: Ack ack!  This sucked...\n");
 	unmap_buffer(bh); iput(inode); return NULL;
 }
+#endif /* CONFIG_FS_RO */
 
 unsigned long minix_count_free_inodes(sb)
 struct super_block *sb;
