@@ -22,13 +22,15 @@ unsigned int origin;
 	off_t offset = get_fs_long(p_offset);
 	struct file * file;
 	int tmp = -1;
-
+	
+/*	printk("%d lseek1\n",origin); */
 	if (fd >= NR_OPEN || !(file=current->files.fd[fd]) || !(file->f_inode))
 		return -EBADF;
 	if (origin > 2)
 		return -EINVAL;
 	if (file->f_op && file->f_op->lseek)
 		return file->f_op->lseek(file->f_inode,file,offset,origin);
+/*	printk("%ld lseek2\n",offset); */
 
 /* this is the default handler if no lseek handler is present */
 	switch (origin) {
@@ -44,6 +46,8 @@ unsigned int origin;
 			tmp = file->f_inode->i_size + offset;
 			break;
 	}
+/*	printk("%d lseek3\n",fd); */
+
 	if (tmp < 0)
 		return -EINVAL;
 	if (tmp != file->f_pos) {
@@ -68,7 +72,11 @@ unsigned int count;
 	if (!(file->f_mode & 1))
 		return -EBADF;
 	if (!file->f_op || !file->f_op->read)
+	{
+		printk("readwrite: we're analphabetic\n");
 		return -EINVAL;
+	}
+
 	if (!count)
 		return 0;
 	error = verify_area(VERIFY_WRITE,buf,count);
