@@ -45,7 +45,10 @@ struct _mminit {
 typedef struct _mminit __arch_mminit;
 typedef struct _mminit * __parch_mminit;
 
+#ifndef _SIZE_T
+#define _SIZE_T
 typedef int size_t;
+#endif
 
 #ifndef NULL
 #define NULL	0
