@@ -8,7 +8,7 @@
  */
 
 #ifndef CONFIG_ROOTDEV
-#define CONFIG_ROOTDEV 0x3c0
+#define CONFIG_ROOTDEV 0x380
 #endif
 
 /*
@@ -19,7 +19,7 @@
 #endif
 
 #ifndef UTS_MACHINE
-#define UTS_MACHINE "unknown"
+#define UTS_MACHINE "i8086"
 #endif
 
 #ifndef UTS_NODENAME
@@ -36,9 +36,9 @@
  */
 
 /* Don't touch these, unless you really know what your doing. */
-#define DEF_INITSEG	0x0200
+#define DEF_INITSEG	0x0100
 #define DEF_SYSSEG	0x1000
-#define DEF_SETUPSEG	0x0220
+#define DEF_SETUPSEG	0x0120
 #define DEF_SYSSIZE	0x2F00
 
 /* internal svga startup constants */
