@@ -1,20 +1,20 @@
-#include <linuxmt/autoconf.h>
 #include <linuxmt/config.h>
 #include <linuxmt/types.h>
 #include <arch/system.h>
 #include <linuxmt/sched.h>
 #include <linuxmt/timex.h>
+#include <linuxmt/utsname.h>
 
 /*
  *	System variable setups
  */
-
+#ifdef CONFIG_FS_RO
+int root_mountflags = 0 | MS_RDONLY;
+#else
 int root_mountflags = 0;
-
+#endif
 /**************************************/
 
-void taska();
-void taskb();
 static void init_task();
 
 extern __ptask _reglasttask, _regnexttask;
@@ -46,27 +46,7 @@ void start_kernel()
 	device_setup();
 	inode_init();
 	fs_init();
-	printk("ELKS version 0.0.58\n");
-/* #define TESTING */
-#ifdef TESTING	
-#if 1
-	while(!key_pressed)
-		printk("Jiffies: %ld\r", jiffies);
-#endif		
-	printk("Pressed code %d (%c)\n\n",key_pressed,key_pressed);
-	key_pressed=0;
-	while(!key_pressed);
-	
-	kfork_proc(&task[1],taska);
-	printk("Got past kfork_proc\n");	
-	/* 
-	 * We are now the idle task. We won't run unless no other
-	 * process can run
-	 */
-	while (1) {
-		schedule();
-	};
-#else
+	printk("ELKS version %s\n", system_utsname.release );
 	task[0].t_kstackm = KSTACK_MAGIC;
 	kfork_proc(&task[1],init_task);
 	/* 
@@ -76,7 +56,6 @@ void start_kernel()
 	while (1) {
 		schedule();
 	}
-#endif		
 }
 
 static char args[] = "/bin/init\0\0";
@@ -89,7 +68,7 @@ static void init_task()
 	printk("Starting init...\n");
 
 	/* Root of /dev/fd0 */
-	ROOT_DEV=CONFIG_ROOTDEV;
+/*	ROOT_DEV=CONFIG_ROOTDEV; */
 	mount_root();
 	
 	printk("Root mounted.\n");
@@ -107,7 +86,7 @@ static void init_task()
 	printk("Loading init\n");
 	if (sys_execve("/bin/init", args, 0 )) {
 		if (sys_execve("/bin/sh", args, 0))
-			panic("panic: No init or sh found");
+			panic("No init or sh found");
 		if (sys_dup(0)!=1)
 			printk("dup failed\n");
 		sys_dup(0);
@@ -168,7 +147,7 @@ _delay:
 	or ax,ax
 	jz  dellow
 axlp:
-	mov bx,#0
+	xor bx,bx
 *
 *	Delay a complete low word loop time
 *
@@ -230,28 +209,3 @@ int calibrate_delay()
 	return -1;
 }
 
-
-void taska()
-{
-	/*
-	 *	Print 'A' and switch tasks 
-	 */
-	 
-	printk("kfork_proc called me\n");
-	kfork_proc(&task[2],taskb);
-	while (1) 
-	{
-		printk("A");
-		schedule();
-	};
-}
-
-void taskb()
-{
-	while (1) 
-	{
-		printk("B");
-		schedule();
-	}
-}
-
