@@ -20,7 +20,7 @@ extern unsigned char can_tswitch;
 extern int lastirq;
 
 int need_resched = 0;
-
+#if 0
 /* This is a diagnostic dump of all currently running tasks... to be run after
  * fork() for now.  Not very intelligible yet */
 void print_tasks()
@@ -33,7 +33,7 @@ void print_tasks()
 		printk("%d %d %d %x %x\n", i, pt->pid, pt->state, pt->t_regs.cs, pt->t_regs.ds); 
 	}
 }
-
+#endif
 /*
  *	Schedule a task. On entry current is the task, which will
  *	vanish quietly for a while and someone elses thread will return
