@@ -85,13 +85,13 @@ struct wait_queue **p;
 	marker = peekw(get_ds(), get_bp() + 2);	
 	__sleep_on(p, TASK_UNINTERRUPTIBLE);
 }
-
+#if 0
 void interruptible_sleep_on(p)
 struct wait_queue **p;
 {
 	__sleep_on(p, TASK_INTERRUPTIBLE);
 }
-
+#endif
 /*
  * Wake up a process. Put it on the run-queue if it's not
  * already there.  The "current" process is always on the
